#include  <stdio.h>

/*-------------- Konstanten -------------------------------------------------*/
#define EPSILON    0.001     /* Fehlerschranke f. Nullstellenberechnung */

/*-------------- guthaben ---------------------------------------------------*/
/* ermittelt Guthaben bei einer sparrate von x nach n Jahren */
double guthaben(long laufzeit, double sparrate, double zinssatz)
{
   long     i;
   double   kapital=0;

   for (i=1; i<=laufzeit; i++)
      kapital = (kapital + sparrate) * (1 + zinssatz/100);
   return(kapital);
}

/*-------------- f ----------------------------------------------------------*/
/* Quasi math. Funktion f, fuer die Nullstelle ermittelt wird */
double f(long laufzeit, long sparziel, double rate, double zinssatz)
{                                                        /* >0: Rate zu hoch */
   return( guthaben(laufzeit,rate,zinssatz) - sparziel );/* <0: Rate zu niedrig */
}

/*-------------- nullstelle -------------------------------------------------*/
double nullstelle(double x_min, double x_max, long laufzeit,
                 long sparziel, double zinssatz)
{
   double   mitte, y_min, y_mitte;
   do {
      mitte   = (x_max + x_min) / 2;
      y_min   = f(laufzeit, sparziel, x_min, zinssatz);
      y_mitte = f(laufzeit, sparziel, mitte, zinssatz);
      if (y_min * y_mitte > 0)
         x_min=mitte;
      else
         x_max=mitte;
   } while ((x_max-x_min) >= EPSILON );

   return((x_min + x_max) / 2);
}

/*-------------- wert_einlesen ----------------------------------------------*/
/* Eingabefunktion fr long int-Werte */
/* str: String, der den Namen des Werte enthaelt */
long wert_einlesen(char* str)
{
   long   wert;

   printf("Bitte %s eingeben: ",str);
   scanf("%ld", &wert);
   return(wert);
}

/*-------------- main -------------------------------------------------------*/
int  main(void)
{
   long    anfangsjahr, endjahr, endguthaben, laufzeit;
   double  zinssatz, proberate, sparrate;

   printf("Ratenrechnung bei vorgegebener Spardauer & Endguthaben\n"
          "======================================================\n\n");
   anfangsjahr = wert_einlesen("Anfangsjahr");
   endjahr     = wert_einlesen("Endjahr");
   endguthaben = wert_einlesen("Endguthaben (ganzzahlig)");
   printf("Bitte Zinssatz eingeben: ");
   scanf("%lf", &zinssatz);

   laufzeit  = endjahr-anfangsjahr;
   proberate = (double)endguthaben/laufzeit; /* absolutes Max., da selbst bei */
                    /* einem Zinssatz von 0% Sparziel nach Laufzeit erreicht */

   sparrate = nullstelle(1, proberate, laufzeit, endguthaben, zinssatz);

   printf("\n\nJaehrliche Sparrate: %.2f!\n\n", sparrate + 0.005);
   return(0);
}
