#include  <stdio.h>
#include  "bildschi.h"
#include  "fehler.h"
#include  "turing.h"

/*------- Konstanten-Definitionen -----------------------------------------*/
#define  RECHTS   +1
#define  LINKS    -1
#define  NICHTS    0

/*------- Modulglobale Variablen (nur innerhalb dieses Moduls verfuegbar) ---*/
static int   zustand=0;        /*-- aktueller Zustand                     */
static char  eingabe_zeichen;  /*-- zuletzt gelesenes Zeichen             */
static int   alt_kopfpos=0;    /*-- Alte Position des Lese-/Schreibkopfes */
static int   neu_kopfpos=0;    /*-- Neue Position des Lese-/Schreibkopfes */

/*------------- regel ---------------------------------------------------*/
static int  regel(int ausgab_zeich, int richtung, int neu_zustand)
{
    int  rueckgabe;
    
    neu_kopfpos += richtung;
    rueckgabe = bildschirm_bewege_kopf(alt_kopfpos, neu_kopfpos, 
                                       ausgab_zeich, &eingabe_zeichen,
                                       zustand, neu_zustand);
    alt_kopfpos = neu_kopfpos;
    zustand = neu_zustand;
    return(rueckgabe);
}

/*------------- erlaubtes_zeichen ---------------------------------------*/
boolean  erlaubtes_zeichen(char zeichen)
{
   return (zeichen=='0' || zeichen=='1' || zeichen=='#' || zeichen==' ');
}

/*------------- turing_regeln -------------------------------------------*/
int  turing_regeln(void)
{
    if (eingabe_zeichen==' ')  /*  Leerzeichen und # sind identisch */
       eingabe_zeichen='#';
       
    switch (zustand) {
       case  0:
          zustand=1;
          return ( (einaus_init(&eingabe_zeichen)==TRUE) ? OK : FEHLER );

       case  1:
          if (eingabe_zeichen=='0')
             return (regel('0', RECHTS, 1));    /*  100R1 */
          else if (eingabe_zeichen=='1')
             return (regel('1', RECHTS, 1));    /*  111R1 */
          else if (eingabe_zeichen=='#')
             return (regel('#', LINKS, 2));     /*  1##L2 */
          else
             return(FEHLER);

       case  2:
          if (eingabe_zeichen=='0')
             return (regel('1', RECHTS, 4));    /*  201R4 */
          if (eingabe_zeichen=='#')
             return (regel('1', NICHTS, 4));    /*  2#1N4 */
          if (eingabe_zeichen=='1')
             return (regel('0', NICHTS, 3));    /*  210N3 */
          else 
             return(FEHLER);

       case  3:
          if (eingabe_zeichen=='0')
             return (regel('0', LINKS, 2));     /*  300L2 */
          else 
             return(FEHLER);

       case  4:
          if (eingabe_zeichen=='0')
             return (regel('0', RECHTS, 4));    /*  400R4 */
          if (eingabe_zeichen=='1')
             return (regel('1', RECHTS, 4));    /*  411R4 */
          if (eingabe_zeichen=='#')
             return (regel('#', NICHTS, 5));    /*  2##N5 */
          else 
             return(FEHLER);

       case  5:
          return(FERTIG);

       default:
          fehler_meld(".....Unerlaubter Zustand %d in turing_regeln...", zustand);
          return(FEHLER);
    }
}
