#include  <stdio.h>
#include  "bildschi.h"
#include  "fehler.h"
#include  "turing.h"

/*------- Konstanten-Definitionen -----------------------------------------*/
#define  RECHTS   +1
#define  LINKS    -1
#define  NICHTS    0

/*------- Modulglobale Variablen (nur innerhalb dieses Moduls verfuegbar) ---*/
static int   zustand=0;        /*-- aktueller Zustand                     */
static char  eingabe_zeichen;  /*-- zuletzt gelesenes Zeichen             */
static int   alt_kopfpos=0;    /*-- Alte Position des Lese-/Schreibkopfes */
static int   neu_kopfpos=0;    /*-- Neue Position des Lese-/Schreibkopfes */

/*------------- regel ---------------------------------------------------*/
static int  regel(int ausgab_zeich, int richtung, int neu_zustand)
{
    int  rueckgabe;
    
    neu_kopfpos += richtung;
    rueckgabe = bildschirm_bewege_kopf(alt_kopfpos, neu_kopfpos, 
                                       ausgab_zeich, &eingabe_zeichen,
                                       zustand, neu_zustand);
    alt_kopfpos = neu_kopfpos;
    zustand = neu_zustand;
    return(rueckgabe);
}

/*------------- erlaubtes_zeichen ---------------------------------------*/
boolean  erlaubtes_zeichen(char zeichen)
{
   return( zeichen=='A' || zeichen=='B' || zeichen=='C' ||
           zeichen=='X' || zeichen=='Y' || zeichen=='Z' || zeichen=='#');
}


/*------------- turing_regeln -------------------------------------------*/
int  turing_regeln(void)
{
    switch (zustand) {
       case  0:
          zustand=1;
          return ( (einaus_init(&eingabe_zeichen)==TRUE) ? OK : FEHLER );

       case  1:
          if (eingabe_zeichen=='A')
             return (regel('X', RECHTS, 2) );   /*  1axR2 */
          else if (eingabe_zeichen=='Y')
             return (regel('Y', RECHTS, 5) );   /*  1yyR5 */
          else
             return(FEHLER);

       case  2:
          if (eingabe_zeichen=='A')
             return (regel('A', RECHTS, 2));    /*  2aaR2 */
          else if (eingabe_zeichen=='Y')
             return (regel('Y', RECHTS, 2));    /*  2yyR2 */
          else if (eingabe_zeichen=='B')
             return (regel('Y', RECHTS, 3));    /*  2byR3 */
          else 
             return(FEHLER);

       case  3:
          if (eingabe_zeichen=='B')
             return (regel('B', RECHTS, 3));    /*  3bbR3 */
          else if (eingabe_zeichen=='Z')
             return (regel('Z', RECHTS, 3));    /*  3zzR3 */
          else if (eingabe_zeichen=='C')
             return (regel('Z', LINKS, 4));     /*  3czL4 */
          else 
             return(FEHLER);

       case  4:
          if (eingabe_zeichen=='A')
             return (regel('A', LINKS, 4));     /*  4aaL4 */
          else if (eingabe_zeichen=='B')
             return (regel('B', LINKS, 4));     /*  4bbL4 */
          else if (eingabe_zeichen=='Y')
             return (regel('Y', LINKS, 4));     /*  4yyL4 */
          else if (eingabe_zeichen=='Z')
             return (regel('Z', LINKS, 4));     /*  4zzL4 */
          else if (eingabe_zeichen=='X')
             return (regel('X', RECHTS, 1));    /*  4xxR1 */
          else 
             return(FEHLER);

       case  5:
          if (eingabe_zeichen=='Z')
             return (regel('Z', RECHTS, 5));    /*  5zzR5 */
          else if (eingabe_zeichen=='Y')
             return (regel('Y', RECHTS, 5));    /*  5yyR5 */
          else if (eingabe_zeichen=='#')
             return (regel('#', LINKS, 6));     /*  5##L6 */
          else 
             return(FEHLER);

       case  6:
          return(FERTIG);

       default:
          fehler_meld(".....Unerlaubter Zustand %d in turing_regeln...", zustand);
          return(FEHLER);
    }
}
