#include  <stdio.h>
#include  <stdlib.h>
#include  <limits.h>
#include  <string.h>
#include  <dirent.h>
#include  <sys/types.h>
#include  <sys/stat.h>
#include  <unistd.h>

/*---- Konstantendefinitionen ----------------------------------------*/
#define  FTW_F    1   /* Datei ist kein Directory */
#define  FTW_D    2   /* Datei ist ein Directory  */
#define  FTW_DNR  3   /* Nichtlesbares Directory  */
#define  FTW_NS   4   /* Datei, auf die stat erfolglos ist */
#define  MAX_PFAD  1000
#define  MAX_ZEILLAENG   2000

/*---- Typdefinitionen -----------------------------------------------*/
typedef int  MEIN_AUSWERT(const char *, const struct stat *, int);

/*---- Variablendefinitionen -----------------------------------------*/
static char           pfadname[MAX_PFAD];
static int            tiefe = 0;
static long int       dateizahl = 0;
static unsigned long  byte_zahl = 0;

/*---- Forward-Funktionsdeklarationen --------------------------------*/
static  MEIN_AUSWERT  mein_auswert;
static  int           baum_abstieg(char *, MEIN_AUSWERT *);
static  int           pfad_behandel(MEIN_AUSWERT *);

/*---- main ----------------------------------------------------------*/
int  main(int argc, char *argv[])
{
   if (argc != 2) {
      fprintf(stderr, "usage:  %s directory\n", argv[0]);
      exit(1);
   }
   return( baum_abstieg(argv[1], mein_auswert) );
}

/*---- baum_abstieg --------------------------------------------------*/
static int  baum_abstieg(char *pfad, MEIN_AUSWERT *funktion)
{
   char   dirname[1000];
   int    n;

   if (getcwd(dirname, 1000) == NULL) { /* Ermitteln des Working-Directory-Namens */
      fprintf(stderr, "....kann Name des Working-Directorys nicht ermitteln\n");
      exit(2);
   }
   if (chdir(pfad) < 0) {                     /* In angegebenen Pfad wechseln */
      fprintf(stderr, "....kann nicht zu %s wechseln\n", pfad);
      exit(3);
   }
   if (getcwd(pfadname, MAX_PFAD) == NULL) { /* Absoluten Pfadnamen ermitteln */
      fprintf(stderr, "....Fehler bei getcwd fuer %s\n", pfad);
      exit(4);
   }

   n = pfad_behandel(funktion);

   fprintf(stdout, "%s: %lu Bytes (%lu KBytes)\n",
                   pfad, byte_zahl, byte_zahl/1000);

   if (chdir(dirname) < 0) {   /*----- Wechseln in ursprgl. Directory */
      fprintf(stderr, ".... kann nicht in urspruengliches Directory zurueckwechseln\n");
      exit(5);
   }
   return(n);
}

/*---- pfad_behandel -------------------------------------------------*/
static int  pfad_behandel(MEIN_AUSWERT *funktion)
{
   struct stat     statpuff;
   struct dirent   *direntz;
   DIR             *dirz;
   int             n;
   char            *zgr;

   if (stat(pfadname, &statpuff) < 0)
      return(funktion(pfadname, &statpuff, FTW_NS)); /* Fehler bei stat */

   if ((statpuff.st_mode & S_IFDIR) == 0)
      return(funktion(pfadname, &statpuff, FTW_F));  /* kein Directory */

   /* Es liegt ein Directory vor, fuer das zuerst funktion()
    * aufgerufen wird, bevor jeder einzelne Dateiname dieses Directorys
    * bearbeitet wird.                                                  */
   if ( (dirz = opendir(pfadname)) == NULL) { /* Directory nicht lesbar */
      closedir(dirz);
      return(funktion(pfadname, &statpuff, FTW_DNR));
   }

   if ( (n = funktion(pfadname, &statpuff, FTW_D)) != 0) /* Ausg. von Directorypfad */
      return(n);

   zgr = pfadname + strlen(pfadname);   /* Slash an Pfadnamen anhaengen */
   *zgr++ = '/';
   *zgr = '\0';

   while ( (direntz = readdir(dirz)) != NULL) {
         /* . und .. ignorieren */
      if (strcmp(direntz->d_name, ".") && strcmp(direntz->d_name, "..")) {
         strcpy(zgr, direntz->d_name); /* Dateinamen nach Slash anhaengen */
         tiefe++;
         if (pfad_behandel(funktion) != 0) {   /* Rekursion */
            tiefe--;
            break;
         }
         tiefe--;
      }
   }
   *(zgr-1) = '\0';     /* Nach Slash alles wieder loeschen */

   if (closedir(dirz) < 0)
      fprintf(stderr, "....closedir fuer %s schlug fehl", pfadname);

   return(n);
}

/*---- mein_auswert --------------------------------------------------*/
static int  mein_auswert(const char *pfad, const struct stat *statzgr, 
                         int dateityp)
{
   int  i;

   switch (dateityp) {
         case FTW_F:
         case FTW_D:
            byte_zahl += statzgr->st_size;
            break;
         case FTW_DNR:
            fprintf(stderr, "..Directory %s nicht lesbar\n", pfad);
            break;
         case FTW_NS:
            fprintf(stderr, "....Fehler bei stat auf Datei %s\n", pfad);
            break;
         default:
            fprintf(stderr, "....Unbekannter Dateityp (%d) bei Datei %s\n", dateityp, pfad);
            exit(6);
   }
   return(0);
}
